@@ -1,4 +1,4 @@
-# Copyright (C) 2021-2022 Modin authors
+# Copyright (C) 2021-2023 Modin authors
 #
 # SPDX-License-Identifier: Apache-2.0
 
@@ -18,3 +18,10 @@ class MpiHosts(EnvironmentVariable, type=ExactStr):
     """MPI hosts to run unidist on."""
 
     varname = "UNIDIST_MPI_HOSTS"
+
+
+class MpiPickleThreshold(EnvironmentVariable, type=int):
+    """Minimum buffer size for serialization with pickle 5 protocol"""
+
+    default = 1024**2 // 4  # 0.25 MiB
+    varname = "UNIDIST_MPI_PICKLE_THRESHOLD"
