@@ -1,4 +1,4 @@
-# Copyright (C) 2021-2022 Modin authors
+# Copyright (C) 2021-2023 Modin authors
 #
 # SPDX-License-Identifier: Apache-2.0
 
@@ -24,6 +24,7 @@ import unidist.core.backends.mpi.core.common as common
 # TODO: Find a way to move this after all imports
 mpi4py.rc(recv_mprobe=False, initialize=False)
 from mpi4py import MPI  # noqa: E402
+from mpi4py.util import pkl5  # noqa: E402
 
 
 # Sleep time setting inside the busy wait loop
@@ -170,6 +171,12 @@ def mpi_send_object(comm, data, dest_rank):
         Data to send.
     dest_rank : int
         Target MPI process to transfer data.
+
+    Notes
+    -----
+    This blocking send is used when we have to wait for completion of the communication,
+    which is necessary for the pipeline to continue, or when the receiver is waiting for a result.
+    Otherwise, use non-blocking ``mpi_isend_object``.
     """
     comm.send(data, dest=dest_rank)
 
@@ -209,6 +216,12 @@ def mpi_send_buffer(comm, buffer_size, buffer, dest_rank):
         Buffer object to send.
     dest_rank : int
         Target MPI process to transfer buffer.
+
+    Notes
+    -----
+    This blocking send is used when we have to wait for completion of the communication,
+    which is necessary for the pipeline to continue, or when the receiver is waiting for a result.
+    Otherwise, use non-blocking ``mpi_isend_buffer``.
     """
     comm.send(buffer_size, dest=dest_rank)
     comm.Send([buffer, MPI.CHAR], dest=dest_rank)
@@ -236,7 +249,7 @@ def mpi_recv_buffer(comm, source_rank):
     return s_buffer
 
 
-def mpi_isend_buffer(comm, data, dest_rank):
+def mpi_isend_buffer(comm, buffer_size, buffer, dest_rank):
     """
     Send buffer object to another MPI rank in a non-blocking way.
 
@@ -244,7 +257,9 @@ def mpi_isend_buffer(comm, data, dest_rank):
     ----------
     comm : object
         MPI communicator object.
-    data : object
+    buffer_size : int
+        Buffer size in bytes.
+    buffer : object
         Buffer object to send.
     dest_rank : int
         Target MPI process to transfer data.
@@ -254,7 +269,12 @@ def mpi_isend_buffer(comm, data, dest_rank):
     object
         A handler to MPI_Isend communication result.
     """
-    return comm.Isend([data, MPI.CHAR], dest=dest_rank)
+    requests = []
+    h1 = comm.isend(buffer_size, dest=dest_rank)
+    requests.append((h1, None))
+    h2 = comm.Isend([buffer, MPI.CHAR], dest=dest_rank)
+    requests.append((h2, buffer))
+    return requests
 
 
 def mpi_busy_wait_recv(comm, source_rank):
@@ -311,7 +331,7 @@ def recv_operation_type(comm):
 # --------------------------------- #
 
 
-def _send_complex_data_impl(comm, s_data, raw_buffers, len_buffers, dest_rank):
+def _send_complex_data_impl(comm, s_data, raw_buffers, buffer_count, dest_rank):
     """
     Send already serialized complex data.
 
@@ -323,24 +343,29 @@ def _send_complex_data_impl(comm, s_data, raw_buffers, len_buffers, dest_rank):
         Serialized data as bytearray.
     raw_buffers : list
         Pickle buffers list, out-of-band data collected with pickle 5 protocol.
-    len_buffers : list
-        Size of each buffer from `raw_buffers` list.
+    buffer_count : list
+        List of the number of buffers for each object
+        to be serialized/deserialized using the pickle 5 protocol.
+        See details in :py:class:`~unidist.core.backends.mpi.core.serialization.ComplexDataSerializer`.
     dest_rank : int
         Target MPI process to transfer data.
     """
-    # Send message pack bytestring
-    mpi_send_buffer(comm, len(s_data), s_data, dest_rank)
-    # Send the necessary metadata
-    mpi_send_object(comm, len(raw_buffers), dest_rank)
-    for raw_buffer in raw_buffers:
-        mpi_send_buffer(comm, len(raw_buffer.raw()), raw_buffer, dest_rank)
-    # TODO: do not send if raw_buffers is zero
-    mpi_send_object(comm, len_buffers, dest_rank)
+    info = {
+        "s_data_len": len(s_data),
+        "buffer_count": buffer_count,
+        "raw_buffers_len": [len(sbuf) for sbuf in raw_buffers],
+    }
+
+    comm.send(info, dest=dest_rank)
+    with pkl5._bigmpi as bigmpi:
+        comm.Send(bigmpi(s_data), dest=dest_rank)
+        for sbuf in raw_buffers:
+            comm.Send(bigmpi(sbuf), dest=dest_rank)
 
 
 def send_complex_data(comm, data, dest_rank):
     """
-    Send the data that consists of different user provided complex types, lambdas and buffers.
+    Send the data that consists of different user provided complex types, lambdas and buffers in a blocking way.
 
     Parameters
     ----------
@@ -359,22 +384,28 @@ def send_complex_data(comm, data, dest_rank):
         A list of pickle buffers.
     list
         A list of buffers amount for each object.
+
+    Notes
+    -----
+    This blocking send is used when we have to wait for completion of the communication,
+    which is necessary for the pipeline to continue, or when the receiver is waiting for a result.
+    Otherwise, use non-blocking ``isend_complex_data``.
     """
     serializer = ComplexDataSerializer()
     # Main job
     s_data = serializer.serialize(data)
     # Retrive the metadata
     raw_buffers = serializer.buffers
-    len_buffers = serializer.len_buffers
+    buffer_count = serializer.buffer_count
 
     # MPI comminucation
-    _send_complex_data_impl(comm, s_data, raw_buffers, len_buffers, dest_rank)
+    _send_complex_data_impl(comm, s_data, raw_buffers, buffer_count, dest_rank)
 
     # For caching purpose
-    return s_data, raw_buffers, len_buffers
+    return s_data, raw_buffers, buffer_count
 
 
-def _isend_complex_data_impl(comm, s_data, raw_buffers, len_buffers, dest_rank):
+def _isend_complex_data_impl(comm, s_data, raw_buffers, buffer_count, dest_rank):
     """
     Send serialized complex data.
 
@@ -388,8 +419,10 @@ def _isend_complex_data_impl(comm, s_data, raw_buffers, len_buffers, dest_rank):
         A serialized msgpack data.
     raw_buffers : list
         A list of pickle buffers.
-    len_buffers : list
-        A list of buffers amount for each object.
+    buffer_count : list
+        List of the number of buffers for each object
+        to be serialized/deserialized using the pickle 5 protocol.
+        See details in :py:class:`~unidist.core.backends.mpi.core.serialization.ComplexDataSerializer`.
     dest_rank : int
         Target MPI process to transfer data.
 
@@ -399,31 +432,28 @@ def _isend_complex_data_impl(comm, s_data, raw_buffers, len_buffers, dest_rank):
         A list of pairs, ``MPI_Isend`` handler and associated data to send.
     """
     handlers = []
+    info = {
+        "s_data_len": len(s_data),
+        "buffer_count": buffer_count,
+        "raw_buffers_len": [len(sbuf) for sbuf in raw_buffers],
+    }
 
-    # Send message pack bytestring
-    h1 = mpi_isend_object(comm, len(s_data), dest_rank)
-    h2 = mpi_isend_buffer(comm, s_data, dest_rank)
+    h1 = comm.isend(info, dest=dest_rank)
     handlers.append((h1, None))
-    handlers.append((h2, s_data))
 
-    # Send the necessary metadata
-    h3 = mpi_isend_object(comm, len(raw_buffers), dest_rank)
-    handlers.append((h3, None))
-    for raw_buffer in raw_buffers:
-        h4 = mpi_isend_object(comm, len(raw_buffer.raw()), dest_rank)
-        h5 = mpi_isend_buffer(comm, raw_buffer, dest_rank)
-        handlers.append((h4, None))
-        handlers.append((h5, raw_buffer))
-    # TODO: do not send if raw_buffers is zero
-    h6 = mpi_isend_object(comm, len_buffers, dest_rank)
-    handlers.append((h6, len_buffers))
+    with pkl5._bigmpi as bigmpi:
+        h2 = comm.Isend(bigmpi(s_data), dest=dest_rank)
+        handlers.append((h2, s_data))
+        for sbuf in raw_buffers:
+            h_sbuf = comm.Isend(bigmpi(sbuf), dest=dest_rank)
+            handlers.append((h_sbuf, sbuf))
 
     return handlers
 
 
-def _isend_complex_data(comm, data, dest_rank):
+def isend_complex_data(comm, data, dest_rank):
     """
-    Send the data that consists of different user provided complex types, lambdas and buffers.
+    Send the data that consists of different user provided complex types, lambdas and buffers in a non-blocking way.
 
     Non-blocking asynchronous interface.
 
@@ -454,14 +484,14 @@ def _isend_complex_data(comm, data, dest_rank):
     s_data = serializer.serialize(data)
     # Retrive the metadata
     raw_buffers = serializer.buffers
-    len_buffers = serializer.len_buffers
+    buffer_count = serializer.buffer_count
 
     # Send message pack bytestring
     handlers.extend(
-        _isend_complex_data_impl(comm, s_data, raw_buffers, len_buffers, dest_rank)
+        _isend_complex_data_impl(comm, s_data, raw_buffers, buffer_count, dest_rank)
     )
 
-    return handlers, s_data, raw_buffers, len_buffers
+    return handlers, s_data, raw_buffers, buffer_count
 
 
 def recv_complex_data(comm, source_rank):
@@ -485,24 +515,19 @@ def recv_complex_data(comm, source_rank):
     # Recv main message pack buffer.
     # First MPI call uses busy wait loop to remove possible contention
     # in a long running data receive operations.
-    buf_size = mpi_busy_wait_recv(comm, source_rank)
-    msgpack_buffer = bytearray(buf_size)
-    comm.Recv([msgpack_buffer, MPI.CHAR], source=source_rank)
 
-    # Recv pickle buffers array for all complex data frames
-    raw_buffers_size = comm.recv(source=source_rank)
-    # Pre-allocate pickle buffers list
-    raw_buffers = [None] * raw_buffers_size
-    for i in range(raw_buffers_size):
-        buf_size = comm.recv(source=source_rank)
-        recv_buffer = bytearray(buf_size)
-        comm.Recv([recv_buffer, MPI.CHAR], source=source_rank)
-        raw_buffers[i] = recv_buffer
-    # Recv len of buffers for each complex data frames
-    len_buffers = comm.recv(source=source_rank)
+    info = comm.recv(source=source_rank)
+
+    msgpack_buffer = bytearray(info["s_data_len"])
+    buffer_count = info["buffer_count"]
+    raw_buffers = list(map(bytearray, info["raw_buffers_len"]))
+    with pkl5._bigmpi as bigmpi:
+        comm.Recv(bigmpi(msgpack_buffer), source=source_rank)
+        for rbuf in raw_buffers:
+            comm.Recv(bigmpi(rbuf), source=source_rank)
 
     # Set the necessary metadata for unpacking
-    deserializer = ComplexDataSerializer(raw_buffers, len_buffers)
+    deserializer = ComplexDataSerializer(raw_buffers, buffer_count)
 
     # Start unpacking
     return deserializer.deserialize(msgpack_buffer)
@@ -513,32 +538,9 @@ def recv_complex_data(comm, source_rank):
 # ---------- #
 
 
-def send_complex_operation(comm, operation_type, operation_data, dest_rank):
-    """
-    Send operation and data that consist of different user provided complex types, lambdas and buffers.
-
-    The data is serialized with ``unidist.core.backends.mpi.core.ComplexDataSerializer``.
-
-    Parameters
-    ----------
-    comm : object
-        MPI communicator object.
-    operation_type : ``unidist.core.backends.mpi.core.common.Operation``
-        Operation message type.
-    operation_data : object
-        Data object to send.
-    dest_rank : int
-        Target MPI process to transfer data.
-    """
-    # Send operation type
-    comm.send(operation_type, dest=dest_rank)
-    # Send complex dictionary data
-    send_complex_data(comm, operation_data, dest_rank)
-
-
 def send_simple_operation(comm, operation_type, operation_data, dest_rank):
     """
-    Send an operation and standard Python data types.
+    Send an operation type and standard Python data types in a blocking way.
 
     Parameters
     ----------
@@ -553,14 +555,51 @@ def send_simple_operation(comm, operation_type, operation_data, dest_rank):
 
     Notes
     -----
-    Serialization is a simple pickle.dump in this case.
+    * This blocking send is used when we have to wait for completion of the communication,
+    which is necessary for the pipeline to continue, or when the receiver is waiting for a result.
+    Otherwise, use non-blocking ``isend_simple_operation``.
+    * Serialization of the data to be sent takes place just using ``pickle.dump`` in this case.
     """
     # Send operation type
     mpi_send_object(comm, operation_type, dest_rank)
-    # Send request details
+    # Send the details of a communication request
     mpi_send_object(comm, operation_data, dest_rank)
 
 
+def isend_simple_operation(comm, operation_type, operation_data, dest_rank):
+    """
+    Send an operation type and standard Python data types in a non-blocking way.
+
+    Parameters
+    ----------
+    comm : object
+        MPI communicator object.
+    operation_type : unidist.core.backends.mpi.core.common.Operation
+        Operation message type.
+    operation_data : object
+        Data object to send.
+    dest_rank : int
+        Target MPI process to transfer data.
+
+    Returns
+    -------
+    list
+        A list of pairs, ``MPI_Isend`` handler and associated data to send.
+
+    Notes
+    -----
+    Serialization of the data to be sent takes place just using ``pickle.dump`` in this case.
+    """
+    # Send operation type
+    handlers = []
+    h1 = mpi_isend_object(comm, operation_type, dest_rank)
+    handlers.append((h1, operation_type))
+    # Send the details of a communication request
+    h2 = mpi_isend_object(comm, operation_data, dest_rank)
+    handlers.append((h2, operation_data))
+    return handlers
+
+
 def recv_simple_operation(comm, source_rank):
     """
     Receive an object of a standard Python data type.
@@ -584,91 +623,6 @@ def recv_simple_operation(comm, source_rank):
     return comm.recv(source=source_rank)
 
 
-def send_operation_data(comm, operation_data, dest_rank, is_serialized=False):
-    """
-    Send data that consists of different user provided complex types, lambdas and buffers.
-
-    The data is serialized with ``unidist.core.backends.mpi.core.ComplexDataSerializer``.
-    Function works with already serialized data.
-
-    Parameters
-    ----------
-    comm : object
-        MPI communicator object.
-    operation_data : object
-        Data object to send.
-    dest_rank : int
-        Target MPI process to transfer data.
-    is_serialized : bool
-        `operation_data` is already serialized or not.
-
-    Returns
-    -------
-    dict or None
-        Serialization data for caching purpose or nothing.
-
-    Notes
-    -----
-    Function returns ``None`` if `operation_data` is already serialized,
-    otherwise ``dict`` containing data serialized in this function.
-    """
-    if is_serialized:
-        # Send already serialized data
-        s_data = operation_data["s_data"]
-        raw_buffers = operation_data["raw_buffers"]
-        len_buffers = operation_data["len_buffers"]
-        _send_complex_data_impl(comm, s_data, raw_buffers, len_buffers, dest_rank)
-        return None
-    else:
-        # Serialize and send the data
-        s_data, raw_buffers, len_buffers = send_complex_data(
-            comm, operation_data, dest_rank
-        )
-        return {
-            "s_data": s_data,
-            "raw_buffers": raw_buffers,
-            "len_buffers": len_buffers,
-        }
-
-
-def send_operation(
-    comm, operation_type, operation_data, dest_rank, is_serialized=False
-):
-    """
-    Send operation and data that consists of different user provided complex types, lambdas and buffers.
-
-    The data is serialized with ``unidist.core.backends.mpi.core.ComplexDataSerializer``.
-    Function works with already serialized data.
-
-    Parameters
-    ----------
-    comm : object
-        MPI communicator object.
-    operation_type : ``unidist.core.backends.mpi.core.common.Operation``
-        Operation message type.
-    operation_data : object
-        Data object to send.
-    dest_rank : int
-        Target MPI process to transfer data.
-    is_serialized : bool
-        `operation_data` is already serialized or not.
-
-    Returns
-    -------
-    dict or None
-        Serialization data for caching purpose.
-
-    Notes
-    -----
-    Function returns ``None`` if `operation_data` is already serialized,
-    otherwise ``dict`` containing data serialized in this function.
-    """
-    # Send operation type
-    mpi_send_object(comm, operation_type, dest_rank)
-    # Send operation data
-    return send_operation_data(comm, operation_data, dest_rank, is_serialized)
-
-
 def isend_complex_operation(
     comm, operation_type, operation_data, dest_rank, is_serialized=False
 ):
@@ -713,49 +667,28 @@ def isend_complex_operation(
         # Send already serialized data
         s_data = operation_data["s_data"]
         raw_buffers = operation_data["raw_buffers"]
-        len_buffers = operation_data["len_buffers"]
+        buffer_count = operation_data["buffer_count"]
 
         h2_list = _isend_complex_data_impl(
-            comm, s_data, raw_buffers, len_buffers, dest_rank
+            comm, s_data, raw_buffers, buffer_count, dest_rank
         )
         handlers.extend(h2_list)
 
         return handlers, None
     else:
         # Serialize and send the data
-        h2_list, s_data, raw_buffers, len_buffers = _isend_complex_data(
+        h2_list, s_data, raw_buffers, buffer_count = isend_complex_data(
             comm, operation_data, dest_rank
         )
         handlers.extend(h2_list)
         return handlers, {
             "s_data": s_data,
             "raw_buffers": raw_buffers,
-            "len_buffers": len_buffers,
+            "buffer_count": buffer_count,
         }
 
 
-def send_remote_task_operation(comm, operation_type, operation_data, dest_rank):
-    """
-    Send operation and data that consist of different user provided complex types, lambdas and buffers.
-
-    Parameters
-    ----------
-    comm : object
-        MPI communicator object.
-    operation_type : ``unidist.core.backends.mpi.core.common.Operation``
-        Operation message type.
-    operation_data : object
-        Data object to send.
-    dest_rank : int
-        Target MPI process to transfer data.
-    """
-    # Send operation type
-    mpi_send_object(comm, operation_type, dest_rank)
-    # Serialize and send the complex data
-    send_complex_data(comm, operation_data, dest_rank)
-
-
-def send_serialized_operation(comm, operation_type, operation_data, dest_rank):
+def isend_serialized_operation(comm, operation_type, operation_data, dest_rank):
     """
     Send operation and serialized simple data.
 
@@ -769,11 +702,20 @@ def send_serialized_operation(comm, operation_type, operation_data, dest_rank):
         Data object to send.
     dest_rank : int
         Target MPI process to transfer data.
+
+    Returns
+    -------
+    list
+        A list of pairs, ``MPI_Isend`` handler and associated data to send.
     """
+    handlers = []
     # Send operation type
-    mpi_send_object(comm, operation_type, dest_rank)
-    # Send request details
-    mpi_send_buffer(comm, len(operation_data), operation_data, dest_rank)
+    h1 = mpi_isend_object(comm, operation_type, dest_rank)
+    handlers.append((h1, operation_type))
+    # Send the details of a communication request
+    h2_list = mpi_isend_buffer(comm, len(operation_data), operation_data, dest_rank)
+    handlers.extend(h2_list)
+    return handlers
 
 
 def recv_serialized_data(comm, source_rank):
