@@ -1,4 +1,4 @@
-# Copyright (C) 2021-2022 Modin authors
+# Copyright (C) 2021-2023 Modin authors
 #
 # SPDX-License-Identifier: Apache-2.0
 
@@ -28,10 +28,13 @@ from unidist.core.backends.mpi.core.controller.common import (
 )
 import unidist.core.backends.mpi.core.common as common
 import unidist.core.backends.mpi.core.communication as communication
+from unidist.core.backends.mpi.core.async_operations import AsyncOperations
 from unidist.config import (
     CpuCount,
     IsMpiSpawnWorkers,
     MpiHosts,
+    ValueSource,
+    MpiPickleThreshold,
 )
 
 
@@ -81,10 +84,23 @@ def init():
     if rank == 0 and parent_comm == MPI.COMM_NULL:
         if IsMpiSpawnWorkers.get():
             nprocs_to_spawn = CpuCount.get() + 1  # +1 for monitor process
-            args = [
-                "-c",
-                "import unidist; import unidist.config as cfg; cfg.Backend.put('mpi'); unidist.init()",
+            args = ["-c"]
+            py_str = [
+                "import unidist",
+                "import unidist.config as cfg",
+                "cfg.Backend.put('mpi')",
             ]
+            if IsMpiSpawnWorkers.get_value_source() != ValueSource.DEFAULT:
+                py_str += [f"cfg.IsMpiSpawnWorkers.put({IsMpiSpawnWorkers.get()})"]
+            if MpiHosts.get_value_source() != ValueSource.DEFAULT:
+                py_str += [f"cfg.MpiHosts.put('{MpiHosts.get()}')"]
+            if CpuCount.get_value_source() != ValueSource.DEFAULT:
+                py_str += [f"cfg.CpuCount.put({CpuCount.get()})"]
+            if MpiPickleThreshold.get_value_source() != ValueSource.DEFAULT:
+                py_str += [f"cfg.MpiPickleThreshold.put({MpiPickleThreshold.get()})"]
+            py_str += ["unidist.init()"]
+            py_str = "; ".join(py_str)
+            args += [py_str]
 
             hosts = MpiHosts.get()
             info = MPI.Info.Create()
@@ -166,8 +182,12 @@ def shutdown():
     mpi_state = communication.MPIState.get_instance()
     # Send shutdown commands to all ranks
     for rank_id in range(communication.MPIRank.MONITOR, mpi_state.world_size):
+        # We use a blocking send here because we have to wait for
+        # completion of the communication, which is necessary for the pipeline to continue.
         communication.mpi_send_object(mpi_state.comm, common.Operation.CANCEL, rank_id)
         logger.debug("Shutdown rank {}".format(rank_id))
+    async_operations = AsyncOperations.get_instance()
+    async_operations.finish()
     if not MPI.Is_finalized():
         MPI.Finalize()
 
@@ -286,6 +306,8 @@ def wait(data_ids, num_returns=1):
 
         operation_type = common.Operation.WAIT
         operation_data = {"id": data_id.base_data_id()}
+        # We use a blocking send here because we have to wait for
+        # completion of the communication, which is necessary for the pipeline to continue.
         communication.send_simple_operation(
             mpi_state.comm,
             operation_type,
@@ -376,12 +398,14 @@ def submit(task, *args, num_returns=1, **kwargs):
         "kwargs": unwrapped_kwargs,
         "output": common.master_data_ids_to_base(output_ids),
     }
-    communication.send_remote_task_operation(
+    async_operations = AsyncOperations.get_instance()
+    h_list, _ = communication.isend_complex_operation(
         communication.MPIState.get_instance().comm,
         operation_type,
         operation_data,
         dest_rank,
     )
+    async_operations.extend(h_list)
 
     # Track the task execution
     garbage_collector.increment_task_counter()
