@@ -1,4 +1,4 @@
-# Copyright (C) 2021-2022 Modin authors
+# Copyright (C) 2021-2023 Modin authors
 #
 # SPDX-License-Identifier: Apache-2.0
 
@@ -8,6 +8,7 @@ import importlib
 import inspect
 import sys
 from collections.abc import KeysView
+from mpi4py.MPI import memory
 
 # Serialization libraries
 if sys.version_info[1] < 8:  # check the minor Python version
@@ -23,6 +24,8 @@ import cloudpickle as cpkl
 import msgpack
 import gc  # msgpack optimization
 
+from unidist.config import MpiPickleThreshold
+
 # Pickle 5 protocol compatible types check
 compatible_modules = ("pandas", "numpy")
 available_modules = []
@@ -70,15 +73,14 @@ def is_pickle5_serializable(data):
     bool
         ``True`` if the data should be serialized with pickle using protocol 5 (out-of-band data).
     """
-    is_serializable = False
     for module in available_modules:
-        if module.__name__ == "pandas":
-            is_serializable = isinstance(data, (module.DataFrame, module.Series))
-            break
-        elif module.__name__ == "numpy":
-            is_serializable = isinstance(data, module.ndarray)
-            break
-    return is_serializable
+        if module.__name__ == "pandas" and isinstance(
+            data, (module.DataFrame, module.Series)
+        ):
+            return True
+        elif module.__name__ == "numpy" and isinstance(data, module.ndarray):
+            return True
+    return False
 
 
 class ComplexDataSerializer:
@@ -89,17 +91,27 @@ class ComplexDataSerializer:
     ----------
     buffers : list, default: None
         A list of ``PickleBuffer`` objects for data decoding.
-    len_buffers : list, default: None
-        A list of buffer sizes for data decoding.
+    buffer_count : list, default: None
+        List of the number of buffers for each object
+        to be serialized/deserialized using the pickle 5 protocol.
 
     Notes
     -----
-    Uses a combination of msgpack, cloudpickle and pickle libraries
+    Uses a combination of msgpack, cloudpickle and pickle libraries.
+    Msgpack allows to serialize/deserialize internal objects of a container separately,
+    but send them as one object. For example, for an array of pandas DataFrames,
+    each DataFrame will be serialized separately using pickle 5,
+    and all `buffers` will be stored in one array to be sent together.
+    To deserialize it `buffer_count` is used, which contains information
+    about the number of `buffers` for each internal object.
     """
 
-    def __init__(self, buffers=None, len_buffers=None):
+    # Minimum buffer size for serialization with pickle 5 protocol
+    PICKLE_THRESHOLD = MpiPickleThreshold.get()
+
+    def __init__(self, buffers=None, buffer_count=None):
         self.buffers = buffers if buffers else []
-        self.len_buffers = len_buffers if len_buffers else []
+        self.buffer_count = buffer_count if buffer_count else []
         self._callback_counter = 0
 
     def _buffer_callback(self, pickle_buffer):
@@ -111,9 +123,12 @@ class ComplexDataSerializer:
         pickle_buffer: pickle.PickleBuffer
             Pickle library buffer wrapper.
         """
-        self.buffers.append(pickle_buffer)
-        self._callback_counter += 1
-        return False
+        pickle_buffer = memory(pickle_buffer)
+        if len(pickle_buffer) >= self.PICKLE_THRESHOLD:
+            self.buffers.append(pickle_buffer)
+            self._callback_counter += 1
+            return False
+        return True
 
     def _dataframe_encode(self, frame):
         """
@@ -130,7 +145,7 @@ class ComplexDataSerializer:
             Dictionary with array of serialized bytes.
         """
         s_frame = pkl.dumps(frame, protocol=5, buffer_callback=self._buffer_callback)
-        self.len_buffers.append(self._callback_counter)
+        self.buffer_count.append(self._callback_counter)
         self._callback_counter = 0
         return {"__pickle5_custom__": True, "as_bytes": s_frame}
 
@@ -197,7 +212,7 @@ class ComplexDataSerializer:
         -----
         Uses msgpack, cloudpickle and pickle libraries.
         """
-        return msgpack.packb(data, default=self._encode_custom)
+        return msgpack.packb(data, default=self._encode_custom, strict_types=True)
 
     def _decode_custom(self, obj):
         """
@@ -216,7 +231,7 @@ class ComplexDataSerializer:
             return pkl.loads(obj["as_bytes"])
         elif "__pickle5_custom__" in obj:
             frame = pkl.loads(obj["as_bytes"], buffers=self.buffers)
-            del self.buffers[: self.len_buffers.pop(0)]
+            del self.buffers[: self.buffer_count.pop(0)]
             return frame
         else:
             return obj
@@ -235,7 +250,9 @@ class ComplexDataSerializer:
         Uses msgpack, cloudpickle and pickle libraries.
         """
         gc.disable()  # Performance optimization for msgpack
-        unpacked_data = msgpack.unpackb(s_data, object_hook=self._decode_custom)
+        unpacked_data = msgpack.unpackb(
+            s_data, object_hook=self._decode_custom, strict_map_key=False
+        )
         gc.enable()
         return unpacked_data
 
