@@ -1,4 +1,4 @@
-# Copyright (C) 2021-2022 Modin authors
+# Copyright (C) 2021-2023 Modin authors
 #
 # SPDX-License-Identifier: Apache-2.0
 
@@ -18,7 +18,7 @@ import unidist.core.backends.mpi.core.communication as communication
 from unidist.core.backends.mpi.core.worker.object_store import ObjectStore
 from unidist.core.backends.mpi.core.worker.request_store import RequestStore
 from unidist.core.backends.mpi.core.worker.task_store import TaskStore
-from unidist.core.backends.mpi.core.worker.async_operations import AsyncOperations
+from unidist.core.backends.mpi.core.async_operations import AsyncOperations
 
 # TODO: Find a way to move this after all imports
 mpi4py.rc(recv_mprobe=False, initialize=False)
@@ -28,9 +28,9 @@ mpi_state = communication.MPIState.get_instance()
 # Logger configuration
 # When building documentation we do not have MPI initialized so
 # we use the condition to set "worker_0.log" in order to build it succesfully.
-log_file = "worker_{}.log".format(mpi_state.rank if mpi_state is not None else 0)
-w_logger = common.get_logger("worker", log_file)
-
+logger_name = "worker_{}".format(mpi_state.rank if mpi_state is not None else 0)
+log_file = "{}.log".format(logger_name)
+w_logger = common.get_logger(logger_name, log_file)
 # Actors map {handle : actor}
 actor_map = {}
 
@@ -106,6 +106,7 @@ async def worker_loop():
 
         elif operation_type == common.Operation.GET:
             request = communication.recv_simple_operation(mpi_state.comm, source_rank)
+            request["id"] = object_store.get_unique_data_id(request["id"])
             request_store.process_get_request(
                 request["source"], request["id"], request["is_blocking_op"]
             )
@@ -115,6 +116,7 @@ async def worker_loop():
             w_logger.debug(
                 "PUT (RECV) {} id from {} rank".format(request["id"]._id, source_rank)
             )
+            request["id"] = object_store.get_unique_data_id(request["id"])
             object_store.put(request["id"], request["data"])
 
             # Clear cached request to another worker, if data_id became available
@@ -127,6 +129,7 @@ async def worker_loop():
 
         elif operation_type == common.Operation.PUT_OWNER:
             request = communication.recv_simple_operation(mpi_state.comm, source_rank)
+            request["id"] = object_store.get_unique_data_id(request["id"])
             object_store.put_data_owner(request["id"], request["owner"])
 
             w_logger.debug(
@@ -138,6 +141,7 @@ async def worker_loop():
         elif operation_type == common.Operation.WAIT:
             request = communication.recv_simple_operation(mpi_state.comm, source_rank)
             w_logger.debug("WAIT for {} id".format(request["id"]._id))
+            request["id"] = object_store.get_unique_data_id(request["id"])
             request_store.process_wait_request(request["id"])
 
         elif operation_type == common.Operation.ACTOR_CREATE:
