@@ -1,4 +1,4 @@
-# Copyright (C) 2021-2022 Modin authors
+# Copyright (C) 2021-2023 Modin authors
 #
 # SPDX-License-Identifier: Apache-2.0
 
@@ -10,19 +10,17 @@ import time
 from unidist.core.backends.common.data_id import is_data_id
 import unidist.core.backends.mpi.core.common as common
 import unidist.core.backends.mpi.core.communication as communication
+from unidist.core.backends.mpi.core.async_operations import AsyncOperations
 from unidist.core.backends.mpi.core.worker.object_store import ObjectStore
 from unidist.core.backends.mpi.core.worker.request_store import RequestStore
 
-
+mpi_state = communication.MPIState.get_instance()
 # Logger configuration
 # When building documentation we do not have MPI initialized so
 # we use the condition to set "worker_0.log" in order to build it succesfully.
-log_file = "worker_{}.log".format(
-    communication.MPIState.get_instance().rank
-    if communication.MPIState.get_instance()
-    else 0
-)
-w_logger = common.get_logger("worker", log_file)
+logger_name = "worker_{}".format(mpi_state.rank if mpi_state is not None else 0)
+log_file = "{}.log".format(logger_name)
+w_logger = common.get_logger(logger_name, log_file)
 
 
 class TaskStore:
@@ -136,12 +134,14 @@ class TaskStore:
             "id": data_id,
             "is_blocking_op": False,
         }
-        communication.send_simple_operation(
+        async_operations = AsyncOperations.get_instance()
+        h_list = communication.isend_simple_operation(
             communication.MPIState.get_instance().comm,
             operation_type,
             operation_data,
             dest_rank,
         )
+        async_operations.extend(h_list)
 
         # Save request in order to prevent massive communication during pending task checks
         RequestStore.get_instance().put(data_id, dest_rank, RequestStore.REQ_DATA_CACHE)
@@ -168,14 +168,16 @@ class TaskStore:
         If the data ID could not be resolved, the function returns ``True``.
         """
         if is_data_id(arg):
-            if ObjectStore.get_instance().contains(arg):
+            object_store = ObjectStore.get_instance()
+            arg = object_store.get_unique_data_id(arg)
+            if object_store.contains(arg):
                 value = ObjectStore.get_instance().get(arg)
                 # Data is already local or was pushed from master
                 return value, False
-            elif ObjectStore.get_instance().contains_data_owner(arg):
+            elif object_store.contains_data_owner(arg):
                 if not RequestStore.get_instance().is_already_requested(arg):
                     # Request the data from an owner worker
-                    owner_rank = ObjectStore.get_instance().get_data_owner(arg)
+                    owner_rank = object_store.get_data_owner(arg)
                     if owner_rank != communication.MPIState.get_instance().rank:
                         self.request_worker_data(owner_rank, arg)
                 return arg, True
@@ -203,6 +205,7 @@ class TaskStore:
         -----
         Exceptions are stored in output data IDs as value.
         """
+        object_store = ObjectStore.get_instance()
         if inspect.iscoroutinefunction(task):
 
             async def execute():
@@ -235,9 +238,11 @@ class TaskStore:
                         and len(output_data_ids) > 1
                     ):
                         for output_id in output_data_ids:
-                            ObjectStore.get_instance().put(output_id, e)
+                            data_id = object_store.get_unique_data_id(output_id)
+                            object_store.put(data_id, e)
                     else:
-                        ObjectStore.get_instance().put(output_data_ids, e)
+                        data_id = object_store.get_unique_data_id(output_data_ids)
+                        object_store.put(data_id, e)
                 else:
                     if output_data_ids is not None:
                         if (
@@ -245,16 +250,18 @@ class TaskStore:
                             and len(output_data_ids) > 1
                         ):
                             for output_id, value in zip(output_data_ids, output_values):
-                                ObjectStore.get_instance().put(output_id, value)
+                                data_id = object_store.get_unique_data_id(output_id)
+                                object_store.put(data_id, value)
                         else:
-                            ObjectStore.get_instance().put(
-                                output_data_ids, output_values
-                            )
+                            data_id = object_store.get_unique_data_id(output_data_ids)
+                            object_store.put(data_id, output_values)
 
                         RequestStore.get_instance().check_pending_get_requests(
                             output_data_ids
                         )
                 # Monitor the task execution
+                # We use a blocking send here because we have to wait for
+                # completion of the communication, which is necessary for the pipeline to continue.
                 communication.mpi_send_object(
                     communication.MPIState.get_instance().comm,
                     common.Operation.TASK_DONE,
@@ -297,9 +304,11 @@ class TaskStore:
                     and len(output_data_ids) > 1
                 ):
                     for output_id in output_data_ids:
-                        ObjectStore.get_instance().put(output_id, e)
+                        data_id = object_store.get_unique_data_id(output_id)
+                        object_store.put(data_id, e)
                 else:
-                    ObjectStore.get_instance().put(output_data_ids, e)
+                    data_id = object_store.get_unique_data_id(output_data_ids)
+                    object_store.put(data_id, e)
             else:
                 if output_data_ids is not None:
                     if (
@@ -307,10 +316,14 @@ class TaskStore:
                         and len(output_data_ids) > 1
                     ):
                         for output_id, value in zip(output_data_ids, output_values):
-                            ObjectStore.get_instance().put(output_id, value)
+                            data_id = object_store.get_unique_data_id(output_id)
+                            object_store.put(data_id, value)
                     else:
-                        ObjectStore.get_instance().put(output_data_ids, output_values)
-            # Monitor the task execution
+                        data_id = object_store.get_unique_data_id(output_data_ids)
+                        object_store.put(data_id, output_values)
+            # Monitor the task execution.
+            # We use a blocking send here because we have to wait for
+            # completion of the communication, which is necessary for the pipeline to continue.
             communication.mpi_send_object(
                 communication.MPIState.get_instance().comm,
                 common.Operation.TASK_DONE,
