@@ -1,4 +1,4 @@
-# Copyright (C) 2021-2022 Modin authors
+# Copyright (C) 2021-2023 Modin authors
 #
 # SPDX-License-Identifier: Apache-2.0
 
@@ -85,8 +85,8 @@ def test_return_none():
 
 
 @pytest.mark.skipif(
-    Backend.get() == BackendName.MP,
-    reason="Run of a remote task inside of another one is not implemented yet for multiprocessing",
+    Backend.get() == BackendName.PYMP,
+    reason="Run of a remote task inside of another one is not implemented yet for pymp",
 )
 def test_internal_remote():
     @unidist.remote
@@ -98,8 +98,8 @@ def test_internal_remote():
 
 
 @pytest.mark.skipif(
-    Backend.get() == BackendName.MP,
-    reason="Serialization of `dict_keys` is not properly implemented yet for multiprocessing",
+    Backend.get() == BackendName.PYMP,
+    reason="Serialization of `dict_keys` is not properly implemented yet for pymp",
 )
 def test_serialize_dict_keys():
     @unidist.remote
@@ -109,3 +109,16 @@ def test_serialize_dict_keys():
     dict_obj = {"1": 1, "2": 2, "3": 3, "4": 4, "5": 5}
 
     assert_equal(f.remote(dict_obj.keys()), dict_obj.keys())
+
+
+def test_serialize_dict_with_tuple_key():
+    @unidist.remote
+    def f(params):
+        return params
+
+    data = {
+        "a": [0, 1, 2, 3],
+        "b": {("a", "b", "c", "d"): "value"},
+    }
+
+    assert_equal(f.remote(data), data)
